#!/usr/bin/env python
#encoding=utf-8

import serial
import sys
import time
import threading

VERSION_MAIN = 1
VERSION_SUB  = 0
VERSION_VAR  = 0

argv = sys.argv
argc = len(argv)

DataDic = dict()	# 子機のSIDと中継機毎のLQI
flag = dict()		# 子機のSIDと出力フラグ
gTime = dict()		# 子機のSIDと結果出力後、最初に受信した時間

# 100ms毎に結果を出力するかどうかを判定する
def OutputDaemon():
	global DataDic
	global flag
	global gTime

	while True:
		nTime = time.time()			# 現在の時刻を取得
		# 1度でも子機の電波を受信した場合
		if len(DataDic) > 0:
			# 子機ごとに処理を行う
			for k, v in gTime.items():
				# 子機の受信時間から1s以上経過、かつ出力フラグが立っている場合、結果を出力する。
				if nTime-v > 1.0 and flag[k]:
					print k + " : ",	# 子機のSIDの出力
					l = 1
					# LQIを降順にソートし、出力する。
					for i, j in sorted( DataDic[k].items(), key=lambda x:x[1], reverse=True ):
						print "%d. %s %3d" % ( l, i, j ),

						if l < len(DataDic[k]):
							l += 1
							print ", ",

					print ""

					DataDic[k].clear()	# 辞書をクリアする
					flag[k] = False		# 出力フラグをFalseにする

		# 100msの間処理を止める
		time.sleep(0.1)

# SWING用の並びになっているか確認する
# 要素の並びと内容が一致したらTWE-EH-SWINGからのパケットと判断
def IS_SWING( mat ):
	ans = [ 'rc', 'lq', 'ct', 'ed', 'id', 'ba', 'bt' ]

	i = 0
	while i < len(mat):
		if ans[i] != mat[i][0]:
			return False
		i += 1

	return True

#	UARTの出力を読み込む
def readSerial( ser, mline ):
	#	UARTを読み込んでリスト化(配列に代入)
	line = ser.readline()
	field = line.strip()			
	field = field.replace("\0", "") #   ヌル文字を消す
	sline = field.split(':')
	length = len(sline)

	# 必要ない部分は削除する
	if length > 2 and field[0] == ":" and field[1] == ":":
		del sline[1]
		del sline[0]
		length -= 2
	else:
		return False

	# 要素数の数で枝刈りする
	if length == 7:
		# listの要素もリスト化する
		i = 0
		while i < 7:
			mline[i] = sline[i].split('=')
			i += 1

		# TWE-EH-SWINGのパケットだった場合、処理する
		if IS_SWING( mline ):
			i = 0
			# 各数字を数値に変換
			while i < 7:
				# 下記パラメータの場合16進数として変換
				# DIのパラメータは16進数、中継機のSIDと子機のSIDは文字列、それ以外は10進数とする
				if mline[i][0] == "bt":
					mline[i][1] = int(mline[i][1], 16)
				elif mline[i][0] == "ed" or mline[i][0] == "rc":
					mline[i][1] = mline[i][1][1:]
				else:
					mline[i][1] = int(mline[i][1])
				i += 1

			return True

		else:
			return False
	else:
		return False

if __name__ == "__main__":
	print "MONOWIRELESS beacon demo Ver." + str(VERSION_MAIN) + "." + str(VERSION_SUB) + "." + str(VERSION_VAR)
	tstart = time.time()

	try:
		#	引数がある場合、引数で指定したデバイスを用いる
		if argc != 2:
			ser = serial.Serial( "/dev/ttyUSB0", 115200, timeout=0.5 );
		else :
			ser = serial.Serial( argv[1], 115200, timeout=0.5 )
	except:
		print "Serial port not found..."
		quit(1)

	# 結果を出力する関数を別スレッドにする
	t = threading.Thread( target=OutputDaemon, args=() )
	t.setDaemon(True)
	t.start()

	# シリアルデータを保存するListの領域確保
	mline = [[0 for i in range(2)] for j in range(7)]

	while True:
		# 現在の時刻を取得する。
		nowTime = time.time()
		try:
			#	シリアルデータを読み込み関数を実行
			bOk = readSerial( ser, mline )
		except KeyboardInterrupt:
			break
		except:
			continue

		# シリアルデータを受け取った場合
		if bOk:
			dic = dict(mline)	# 辞書に変換
			# 一度でも子機からパケットを受け取ったことがある場合
			if DataDic.has_key( dic['ed'] ):
				DataDic[dic['ed']][dic['rc']] = dic['lq']
				# フラグが立っていない場合、フラグを立てて、受信時間を更新する。
				if  flag[dic['ed']] == False:
					gTime[dic['ed']] = nowTime
					flag[dic['ed']] = True
			# 初めて受信する子機の場合、受信時間、受信したモジュールのSID、LQIを保存する
			else:
				DataDic[dic['ed']] = { dic['rc']:dic['lq'] }
				gTime[dic['ed']] = nowTime
				flag[dic['ed']] = True

	ser.close()
